@@ -27,24 +27,28 @@ import pkg_resources
 
 
 # Relative imports to PyInstaller modules.
-from .. import HOMEPATH, DEFAULT_DISTPATH, DEFAULT_WORKPATH
-from .. import compat
-from .. import log as logging
-from ..utils.misc import absnormpath, compile_py_files
-from ..compat import is_win, PYDYLIB_NAMES, open_file
-from ..depend import bindepend
-from ..depend.analysis import initialize_modgraph
-from .api import PYZ, EXE, COLLECT, MERGE
-from .datastruct import TOC, Target, Tree, _check_guts_eq
-from .osx import BUNDLE
-from .toc_conversion import DependencyProcessor
-from .utils import _check_guts_toc_mtime, format_binaries_and_datas
-from ..depend.utils import create_py3_base_library, scan_code_for_ctypes
-from ..archive import pyz_crypto
-from ..utils.misc import get_path_to_toplevel_modules, get_unicode_modules, mtime
+from PyInstaller import HOMEPATH, DEFAULT_DISTPATH, DEFAULT_WORKPATH
+from PyInstaller import compat
+from PyInstaller import log as logging
+from PyInstaller.utils.misc import absnormpath, compile_py_files
+from PyInstaller.compat import is_win, PYDYLIB_NAMES
+from PyInstaller.depend import bindepend
+from PyInstaller.depend.analysis import initialize_modgraph
+from PyInstaller.building.api import PYZ, EXE, COLLECT, MERGE
+from PyInstaller.building.datastruct import TOC, Target, Tree, _check_guts_eq
+from PyInstaller.building.splash import Splash
+from PyInstaller.building.osx import BUNDLE
+from PyInstaller.building.toc_conversion import DependencyProcessor
+from PyInstaller.building.utils import \
+    _check_guts_toc_mtime, format_binaries_and_datas
+from PyInstaller.depend.utils import \
+    create_py3_base_library, scan_code_for_ctypes
+from PyInstaller.archive import pyz_crypto
+from PyInstaller.utils.misc import \
+    get_path_to_toplevel_modules, get_unicode_modules, mtime
 
 if is_win:
-    from ..utils.win32 import winmanifest
+    from PyInstaller.utils.win32 import winmanifest
 
 logger = logging.getLogger(__name__)
 
@@ -127,8 +131,9 @@ class Analysis(Target):
     }
 
     def __init__(self, scripts, pathex=None, binaries=None, datas=None,
-                 hiddenimports=None, hookspath=None, excludes=None, runtime_hooks=None,
-                 cipher=None, win_no_prefer_redirects=False, win_private_assemblies=False,
+                 hiddenimports=None, hookspath=None, hooksconfig=None,
+                 excludes=None, runtime_hooks=None, cipher=None,
+                 win_no_prefer_redirects=False, win_private_assemblies=False,
                  noarchive=False):
         """
         scripts
@@ -144,6 +149,9 @@ class Analysis(Target):
         hookspath
                 An optional list of additional paths to search for hooks.
                 (hook-modules).
+        hooksconfig
+                An optional dict of config settings for hooks.
+                (hook-modules).
         excludes
                 An optional list of module or package names (their Python names,
                 not path names) that will be ignored (as though they were not found).
@@ -164,7 +172,7 @@ class Analysis(Target):
                 individual files.
         """
         super(Analysis, self).__init__()
-        from ..config import CONF
+        from PyInstaller.config import CONF
 
         self.inputs = []
         spec_dir = os.path.dirname(CONF['spec'])
@@ -211,6 +219,10 @@ class Analysis(Target):
                 'pyinstaller40', 'hook-dirs'):
             self.hookspath += list(entry_point.load()())
 
+        self.hooksconfig = {}
+        if hooksconfig:
+            self.hooksconfig.update(hooksconfig)
+
         # Custom runtime hook files that should be included and started before
         # any existing PyInstaller runtime hooks.
         self.custom_runtime_hooks = runtime_hooks or []
@@ -220,7 +232,7 @@ class Analysis(Target):
             # Create a Python module which contains the decryption key which will
             # be used at runtime by pyi_crypto.PyiBlockCipher.
             pyi_crypto_key_path = os.path.join(CONF['workpath'], 'pyimod00_crypto_key.py')
-            with open_file(pyi_crypto_key_path, 'w', encoding='utf-8') as f:
+            with open(pyi_crypto_key_path, 'w', encoding='utf-8') as f:
                 f.write('# -*- coding: utf-8 -*-\n'
                         'key = %r\n' % cipher.key)
             self.hiddenimports.append('tinyaes')
@@ -258,6 +270,7 @@ class Analysis(Target):
             ('pathex', _check_guts_eq),
             ('hiddenimports', _check_guts_eq),
             ('hookspath', _check_guts_eq),
+            ('hooksconfig', _check_guts_eq),
             ('excludes', _check_guts_eq),
             ('custom_runtime_hooks', _check_guts_eq),
             ('win_no_prefer_redirects', _check_guts_eq),
@@ -325,7 +338,7 @@ class Analysis(Target):
         self.datas = TOC(data['datas'])
 
         # Store previously found binding redirects in CONF for later use by PKG/COLLECT
-        from ..config import CONF
+        from PyInstaller.config import CONF
         self.binding_redirects = CONF['binding_redirects'] = data['binding_redirects']
 
         return False
@@ -334,7 +347,7 @@ class Analysis(Target):
         """
         This method is the MAIN method for finding all necessary files to be bundled.
         """
-        from ..config import CONF
+        from PyInstaller.config import CONF
 
         for m in self.excludes:
             logger.debug("Excluding module '%s'" % m)
@@ -360,7 +373,7 @@ class Analysis(Target):
 
         logger.info("running Analysis %s", self.tocbasename)
         # Get paths to Python and, in Windows, the manifest.
-        python = sys.executable
+        python = compat.python_executable
         if not is_win:
             # Linux/MacOS: get a real, non-link path to the running Python executable.
             while os.path.islink(python):
@@ -411,13 +424,13 @@ class Analysis(Target):
         # Save the graph nodes of each in sequence.
         for script in self.inputs:
             logger.info("Analyzing %s", script)
-            priority_scripts.append(self.graph.run_script(script))
+            priority_scripts.append(self.graph.add_script(script))
 
         # Analyze the script's hidden imports (named on the command line)
         self.graph.add_hiddenimports(self.hiddenimports)
 
         ### Post-graph hooks.
-        self.graph.process_post_graph_hooks()
+        self.graph.process_post_graph_hooks(self)
 
         # Update 'binaries' TOC and 'datas' TOC.
         deps_proc = DependencyProcessor(self.graph,
@@ -431,13 +444,20 @@ class Analysis(Target):
         ### Look for dlls that are imported by Python 'ctypes' module.
         # First get code objects of all modules that import 'ctypes'.
         logger.info('Looking for ctypes DLLs')
-        ctypes_code_objs = self.graph.get_co_using_ctypes()  # dict like:  {'module1': code_obj, 'module2': code_obj}
+        # dict like:  {'module1': code_obj, 'module2': code_obj}
+        ctypes_code_objs = self.graph.get_code_using("ctypes")
+
         for name, co in ctypes_code_objs.items():
             # Get dlls that might be needed by ctypes.
             logger.debug('Scanning %s for shared libraries or dlls', name)
             ctypes_binaries = scan_code_for_ctypes(co)
             self.binaries.extend(set(ctypes_binaries))
 
+        self.datas.extend(
+            (dest, source, "DATA") for (dest, source) in
+            format_binaries_and_datas(self.graph.metadata_required())
+        )
+
         # Analyze run-time hooks.
         # Run-time hooks has to be executed before user scripts. Add them
         # to the beginning of 'priority_scripts'.
@@ -535,9 +555,9 @@ class Analysis(Target):
                            + 4 * depInfo.tryexcept)
             return '%s (%s)' % (name, IMPORT_TYPES[imptype])
 
-        from ..config import CONF
+        from PyInstaller.config import CONF
         miss_toc = self.graph.make_missing_toc()
-        with open_file(CONF['warnfile'], 'w', encoding='utf-8') as wf:
+        with open(CONF['warnfile'], 'w', encoding='utf-8') as wf:
             wf.write(WARNFILE_HEADER)
             for (n, p, status) in miss_toc:
                 importers = self.graph.get_importers(n)
@@ -551,15 +571,15 @@ class Analysis(Target):
         """Write a xref (in html) and with `--log-level DEBUG` a dot-drawing
         of the graph.
         """
-        from ..config import CONF
-        with open_file(CONF['xref-file'], 'w', encoding='utf-8') as fh:
+        from PyInstaller.config import CONF
+        with open(CONF['xref-file'], 'w', encoding='utf-8') as fh:
             self.graph.create_xref(fh)
             logger.info("Graph cross-reference written to %s", CONF['xref-file'])
         if logger.getEffectiveLevel() > logging.DEBUG:
             return
         # The `DOT language's <https://www.graphviz.org/doc/info/lang.html>`_
         # default character encoding (see the end of the linked page) is UTF-8.
-        with open_file(CONF['dot-file'], 'w', encoding='utf-8') as fh:
+        with open(CONF['dot-file'], 'w', encoding='utf-8') as fh:
             self.graph.graphreport(fh)
             logger.info("Graph drawing written to %s", CONF['dot-file'])
 
@@ -595,7 +615,7 @@ def build(spec, distpath, workpath, clean_build):
     """
     Build the executable according to the created SPEC file.
     """
-    from ..config import CONF
+    from PyInstaller.config import CONF
 
     # Ensure starting tilde and environment variables get expanded in distpath / workpath.
     # '~/path/abc', '${env_var_name}/path/abc/def'
@@ -663,6 +683,7 @@ def build(spec, distpath, workpath, clean_build):
         'MERGE': MERGE,
         'PYZ': PYZ,
         'Tree': Tree,
+        'Splash': Splash,
         # Python modules available for .spec.
         'os': os,
         'pyi_crypto': pyz_crypto,
@@ -670,7 +691,7 @@ def build(spec, distpath, workpath, clean_build):
 
     # Set up module PyInstaller.config for passing some arguments to 'exec'
     # function.
-    from ..config import CONF
+    from PyInstaller.config import CONF
     CONF['workpath'] = workpath
 
     # Execute the specfile. Read it as a binary file...
@@ -709,7 +730,7 @@ def __add_options(parser):
 
 def main(pyi_config, specfile, noconfirm, ascii=False, **kw):
 
-    from ..config import CONF
+    from PyInstaller.config import CONF
     CONF['noconfirm'] = noconfirm
 
     # Some modules are included if they are detected at build-time or
